//+------------------------------------------------------------------+
//|                                                    svmTrader.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#property indicator_buffers 7

//+-------------------------------------------------------------+
//|   ("svmTrader")  ,    
//|      .
//|            
//| (          ). 
//|
//|     SVM    buy/sell
//|        (   
//|     / , 
//|          ).
//|
//|      / ,  
//|    (.. tickets[]).      
//|  ,      
//|    / .
//|      .      
//| ,       
//|    .
//|
//|  ,        
//| ,    .     ,
//|       .
//|
//|  ...
//+------------------------------------------------------------------+


//+-------  Support Vector Machine Learning-------+
//|   #import   
//|           . 
//|  ,      ,   
//|                                         |
//+------------------------------------------------------------------+
#import "svMachineTool.ex5"
enum ENUM_TRADE {BUY,SELL};
enum ENUM_OPTION {OP_MEMORY,OP_MAXCYCLES,OP_TOLERANCE};
int  initSVMachine(void);
void setIndicatorHandles(int handle,int &indicatorHandles[],int offset,int N);
void setParameter(int handle,ENUM_OPTION option,double value);
bool genOutputs(int handle,ENUM_TRADE trade,int StopLoss,int TakeProfit,double duration);
bool genInputs(int handle);
bool setInputs(int handle,double &Inputs[],int nInputs);
bool setOutputs(int handle,bool &Outputs[]);
bool training(int handle);
bool classify(int handle);
bool classify(int handle,int offset);
bool classify(int handle,double &iput[]);
void  deinitSVMachine(void);
#import

#include <Trade\Trade.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\HistoryOrderInfo.mqh>

//+----------------------- --------------------------+
input int            takeProfit=100;      //    
input int            stopLoss=150;        //    
input double         hours=6;             //         
input double         risk_exp=5;          //         
input double         Tolerance_Value=0.1; //     svm (  10%)
input int            N_DataPoints=100;    //      

//+--------------------- -----------------------+
//|        . I
//|        
//|    .                    
//+------------------------------------------------------------------+
int bears_period=13;
int bulls_period=13;
int ATR_period=13;
int mom_period=13;
int MACD_fast_period=12;
int MACD_slow_period=26;
int MACD_signal_period=9;
int Stoch_Kperiod=5;
int Stoch_Dperiod=3;
int Stoch_slowing=3;
int Force_period=13;

//+--------------------- -------------------------+
int       tickets[];
bool       Opn_B,Opn_S;
datetime    New_Time;
int       handleB,handleS;
double       Vol=1;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
  {
   New_Time=0;
   int handles[];ArrayResize(handles,7);
//+------------------------------------------------------------------+
//|       , 
//|          .      int[].    
//|    ,        
//+------------------------------------------------------------------+
   handles[0]=iBearsPower(Symbol(),0,bears_period);
   handles[1]=iBullsPower(Symbol(),0,bulls_period);
   handles[2]=iATR(Symbol(),0,ATR_period);
   handles[3]=iMomentum(Symbol(),0,mom_period,PRICE_TYPICAL);
   handles[4]=iMACD(Symbol(),0,MACD_fast_period,MACD_slow_period,MACD_signal_period,PRICE_TYPICAL);
   handles[5]=iStochastic(Symbol(),0,Stoch_Kperiod,Stoch_Dperiod,Stoch_slowing,MODE_SMA,STO_LOWHIGH);
   handles[6]=iForce(Symbol(),0,Force_period,MODE_SMA,VOLUME_TICK);

//----------,           ----------
   handleB=initSVMachine();            //  SVM     'handleB'
   setIndicatorHandles(handleB,handles,0,N_DataPoints);   //    SVM       
   setParameter(handleB,OP_TOLERANCE,Tolerance_Value);    //       SVM
   genInputs(handleB);               //  ,   
   genOutputs(handleB,BUY,stopLoss,takeProfit,hours);     //         

//----------,           ----------
   handleS=initSVMachine();            //  SVM     'handleS'
   setIndicatorHandles(handleS,handles,0,N_DataPoints);   //    SVM       
   setParameter(handleS,OP_TOLERANCE,Tolerance_Value);    //       SVM
   genInputs(handleS);               //  ,   
   genOutputs(handleS,SELL,stopLoss,takeProfit,hours);   //         

   training(handleB);   //  SVM   
   training(handleS);   //  SVM      
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   deinitSVMachine();   // ,  SVM,    .
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTick()
  {
   if(New_Time!=iTime(Symbol(),0,0)) //        
     {
      New_Time=iTime(Symbol(),0,0);  //  New_Time variable    
      OnBar();                       //  OnBar()
     }
  }
//+------------------------------------------------------------------+
//|     ,     
//|      .
//+------------------------------------------------------------------+
datetime iTime(string symbol,ENUM_TIMEFRAMES timeframe,int index)
  {
   datetime Time[1];
   ArraySetAsSeries(Time,true);
   int copied=CopyTime(symbol,timeframe,index,1,Time);
   return(Time[0]);
  }
//+------------------------------------------------------------------+
//|   ,     .  
//|       stopLoss/takeProfit ,
//|   ,   stopLoss/takeProfit 
//+------------------------------------------------------------------+
void OnTrade()
  {
   COrderInfo order;
   CTrade trade;
   int t;
   for(int i=0;i<OrdersTotal();i++)
     {
      order.SelectByIndex(i);
      t=(int)order.Ticket();
      if(!order.Select(tickets[t]))
        {
         trade.OrderDelete(t);
        }
     }
  }
//+------------------------------------------------------------------+
//|  OnBar   classify()  
//| support vector machine learning.  classify 
//|        ,     
//| SVM           .  
//|     ,   Open_Order
//+------------------------------------------------------------------+
void OnBar(void)
  {
   Opn_B=classify(handleB);
   Opn_S=classify(handleS);

   if(Opn_B || Opn_S) Open_Order();
  }
//+------------------------------------------------------------------+
//|         
//|       .
//+------------------------------------------------------------------+
void Open_Order(void)
  {
   CTrade trade;
   double CoE=Current_Order_Exposure();   //      
   bool executed=false;
   if(Opn_B==true && CoE<=risk_exp) //  ,   Opn_B = true,        risk_exp
     {
      //-------------    ------------------
      trade.SetDeviationInPoints(50);
      executed=trade.PositionOpen(Symbol(),ORDER_TYPE_BUY,Vol,NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_ASK),Digits()),0,0,"");
      if(executed) //     ,   Insert_Stops()   stopLoss  takeProfit
        {
         Insert_Stops(BUY,NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_ASK),Digits()));
         Opn_B=false;
        }
     }
   if(Opn_S==true && CoE<=risk_exp) //  ,   Opn_S = true,        risk_exp
     {
      //-------------    ------------------
      trade.SetDeviationInPoints(50);
      executed=trade.PositionOpen(Symbol(),ORDER_TYPE_SELL,Vol,NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_BID),Digits()),0,0,"");
      if(executed)
        {
         Insert_Stops(SELL,NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_BID),Digits()));
         Opn_S=false;
        }
     }
  }
//+------------------------------------------------------------------+
//|        
//|       
//|       ( 
//|    ,    
//|       ).
//|  ,       
//|      .
//+------------------------------------------------------------------+
double Current_Order_Exposure()
  {
   COrderInfo order;
   double Volume=0.00;
   for(int i=0;i<OrdersTotal();i++)
     {
      order.SelectByIndex(i);
      Volume=Volume+order.VolumeCurrent();
     }
   return(Volume/2);
  }
//+------------------------------------------------------------------+
//|       stopLoss 
//| takeProfit     .
//+------------------------------------------------------------------+
void Insert_Stops(ENUM_TRADE trade,double Price)
  {
   CTrade trade1,trade2,trade3,trade4;
   int ticket1,ticket2;
   if(trade==BUY)
     {
      trade1.SetDeviationInPoints(50);
      trade1.SellStop(Vol,Price-(Pip_Size()*stopLoss),Symbol(),0.0,0.0,ORDER_TIME_GTC,0,"");
      ticket1=(int)trade1.ResultOrder();
      trade2.SetDeviationInPoints(50);
      trade2.SellLimit(Vol,Price+(Pip_Size()*takeProfit),Symbol(),0.0,0.0,ORDER_TIME_GTC,0,"");
      ticket2=(int)trade2.ResultOrder();
      cacheTicketNumber(ticket1,ticket2);
     }
   if(trade==SELL)
     {
      trade3.SetDeviationInPoints(50);
      trade3.BuyStop(Vol,Price+(Pip_Size()*stopLoss),Symbol(),0.0,0.0,ORDER_TIME_GTC,0,"");
      ticket1=(int)trade3.ResultOrder();
      trade4.SetDeviationInPoints(50);
      trade4.BuyLimit(Vol,Price-(Pip_Size()*takeProfit),Symbol(),0.0,0.0,ORDER_TIME_GTC,0,"");
      ticket2=(int)trade4.ResultOrder();
      cacheTicketNumber(ticket1,ticket2);
     }
  }
//+------------------------------------------------------------------+
//|      stopLoss  
//| takeProfit      .   
//|       .
//+------------------------------------------------------------------+
void cacheTicketNumber(int ticket1,int ticket2)
  {
   if(MathMax(ticket1,ticket2)>(ArraySize(tickets)-1)) ArrayResize(tickets,MathMax(ticket1,ticket2)+1);
   tickets[ticket1]=ticket2;
   tickets[ticket2]=ticket1;
  }
//+------------------------------------------------------------------+
//|         .
//+------------------------------------------------------------------+
double Pip_Size(void)
  {
   double v=SymbolInfoDouble(Symbol(),SYMBOL_BID);
   int deci=(int)MathRound(MathAbs(MathRound(MathLog(v/10000)/MathLog(10))));
   double out=NormalizeDouble(MathPow(10,-1*deci),deci);
   return(out);
  }
//+------------------------------------------------------------------+
